@@ -4,7 +4,7 @@ use strict;
 use Carp "croak";
 use CGI;
 use CGI::Wiki::Plugin::Diff;
-use CGI::Wiki::Plugin::Locator::UK;
+use CGI::Wiki::Plugin::Locator::Grid;
 use OpenGuides::CGI;
 use OpenGuides::Template;
 use OpenGuides::Utils;
@@ -13,7 +13,7 @@ use URI::Escape;
 
 use vars qw( $VERSION );
 
-$VERSION = '0.44';
+$VERSION = '0.45';
 
 =head1 NAME
 
@@ -45,7 +45,19 @@ sub new {
     my $wiki = OpenGuides::Utils->make_wiki_object( config => $args{config} );
     $self->{wiki} = $wiki;
     $self->{config} = $args{config};
-    my $locator = CGI::Wiki::Plugin::Locator::UK->new;
+    # Default to British National Grid for historical reasons.
+    my $geo_handler = $self->config->{_}{geo_handler} || 1;
+    my $locator;
+    if ( $geo_handler == 1 ) {
+        $locator = CGI::Wiki::Plugin::Locator::Grid->new(
+                                             x => "os_x",    y => "os_y" );
+    } elsif ( $geo_handler == 2 ) {
+        $locator = CGI::Wiki::Plugin::Locator::Grid->new(
+                                             x => "osie_x",  y => "osie_y" );
+    } else {
+        $locator = CGI::Wiki::Plugin::Locator::Grid->new(
+                                             x => "easting", y => "northing" );
+    }
     $wiki->register_plugin( plugin => $locator );
     $self->{locator} = $locator;
     my $differ = CGI::Wiki::Plugin::Diff->new;
@@ -183,7 +195,6 @@ sub display_node {
                  language      => $config->{_}->{default_language},
                );
 
-
     # We've undef'ed $version above if this is the current version.
     $tt_vars{current} = 1 unless $version;
 
@@ -360,6 +371,43 @@ sub find_within_distance {
     print CGI->redirect( $script_url . "supersearch.cgi?lat=$lat;long=$long;distance_in_metres=$metres" );
 }
 
+=item B<show_backlinks>
+
+  $guide->show_backlinks( id => "Calthorpe Arms" );
+
+As with other methods, parameters C<return_tt_vars> and
+C<return_output> can be used to return these things instead of
+printing the output to STDOUT.
+
+=cut
+
+sub show_backlinks {
+    my ($self, %args) = @_;
+    my $wiki = $self->wiki;
+    my $formatter = $wiki->formatter;
+
+    my @backlinks = $wiki->list_backlinks( node => $args{id} );
+    my @results = map {
+        { url   => CGI->escape($formatter->node_name_to_node_param($_)),
+	  title => CGI->escapeHTML($_)
+        }             } sort @backlinks;
+    my %tt_vars = ( results       => \@results,
+                    num_results   => scalar @results,
+                    not_deletable => 1,
+                    deter_robots  => 1,
+                    not_editable  => 1 );
+    return %tt_vars if $args{return_tt_vars};
+    my $output = OpenGuides::Template->output(
+                                               node    => $args{id},
+                                               wiki    => $wiki,
+                                               config  => $self->config,
+                                               template=>"backlink_results.tt",
+                                               vars    => \%tt_vars,
+                                             );
+    return $output if $args{return_output};
+    print $output;
+}
+
 =item B<show_index>
 
   $guide->show_index(
@@ -532,7 +580,7 @@ return the output instead of printing it to STDOUT.
 
 sub display_rss {
     my ($self, %args) = @_;
-    use Data::Dumper;warn Dumper \%args;
+
     my $return_output = $args{return_output} ? 1 : 0;
 
     my $items = $args{items} || "";
@@ -573,6 +621,30 @@ As with other methods, parameters C<return_tt_vars> and
 C<return_output> can be used to return these things instead of
 printing the output to STDOUT.
 
+The geographical data that you should provide in the L<CGI> object
+depends on the handler you chose in C<wiki.conf>.
+
+=over
+
+=item *
+
+B<British National Grid> - provide either C<os_x> and C<os_y> or
+C<latitude> and C<longitude>; whichever set of data you give, it will
+be converted to the other and both sets will be stored.
+
+=item *
+
+B<Irish National Grid> - provide either C<osie_x> and C<osie_y> or
+C<latitude> and C<longitude>; whichever set of data you give, it will
+be converted to the other and both sets will be stored.
+
+=item *
+
+B<UTM ellipsoid> - provide C<latitude> and C<longitude>; these will be
+converted to easting and northing and both sets of data will be stored.
+
+=back
+
 =cut
 
 sub commit_node {
@@ -687,11 +759,17 @@ removed.
 If C<password> is not supplied then a form for entering the password
 will be displayed.
 
+As with other methods, parameters C<return_tt_vars> and
+C<return_output> can be used to return these things instead of
+printing the output to STDOUT.
+
 =cut
 
 sub delete_node {
     my ($self, %args) = @_;
     my $node = $args{id} or croak "No node ID supplied for deletion";
+    my $return_tt_vars = $args{return_tt_vars} || 0;
+    my $return_output = $args{return_output} || 0;
 
     my %tt_vars = (
                     not_editable  => 1,
@@ -704,11 +782,14 @@ sub delete_node {
 
     if ($password) {
         if ($password ne $self->config->{_}->{admin_pass}) {
-            print $self->process_template(
+            return %tt_vars if $return_tt_vars;
+            my $output = $self->process_template(
                                      id       => $node,
                                      template => "delete_password_wrong.tt",
                                      tt_vars  => \%tt_vars,
                                    );
+            return $output if $return_output;
+            print $output;
         } else {
             $self->wiki->delete_node(
                                       name    => $node,
@@ -717,18 +798,24 @@ sub delete_node {
             # Check whether any versions of this node remain.
             my %check = $self->wiki->retrieve_node( name => $node );
             $tt_vars{other_versions_remain} = 1 if $check{version};
-            print $self->process_template(
+            return %tt_vars if $return_tt_vars;
+            my $output = $self->process_template(
                                      id       => $node,
                                      template => "delete_done.tt",
                                      tt_vars  => \%tt_vars,
                                    );
+            return $output if $return_output;
+            print $output;
         }
     } else {
-        print $self->process_template(
+        return %tt_vars if $return_tt_vars;
+        my $output = $self->process_template(
                                  id       => $node,
                                  template => "delete_confirm.tt",
                                  tt_vars  => \%tt_vars,
                                );
+        return $output if $return_output;
+        print $output;
     }
 }
 
